/**
 * description
 *
 * @author eson
 *2022年7月20日-10:00:05
 */
package com.yuandian.dataflow.statemachine;

import java.time.Instant;

import com.alipay.sofa.jraft.Status;
import com.alipay.sofa.jraft.error.RemotingException;
import com.alipay.sofa.jraft.rpc.InvokeCallback;
import com.google.protobuf.Any;
import com.yuandian.dataflow.controller.PacketsProcessor.PacketsRequest;
import com.yuandian.dataflow.proto.msgtype.BacktrackingFlowOuterClass;
import com.yuandian.dataflow.statemachine.closure.GenericClosure;
import com.yuandian.dataflow.statemachine.operate.Operate;
import com.yuandian.dataflow.statemachine.operate.Operate.OperateType;
import com.yuandian.dataflow.statemachine.state.State;
import com.yuandian.dataflow.statemachine.state.WorkerState;

import lombok.Getter;
import lombok.Setter;
import lombok.ToString;
import lombok.var;
import lombok.extern.slf4j.Slf4j;

/**
 * Master主线程, 用于接收packets
 *
 * @author eson
 *         2022年7月20日-10:00:05
 */
@Slf4j
@Getter
@Setter
@ToString
public class MasterFactory {

    public static final int MAX_TASKS = 100;

    public static Thread masterExecute = new Thread(new Runnable() {
        @Override
        public void run() {
            try {
                while (true) {
                    log.debug("master execute {}", StateServerFactory.getServerId());
                    var alivePeers = StateServerFactory.getRaftNode().listAlivePeers();
                    log.debug("master execute {}", StateServerFactory.getRaftNode().listAlivePeers());
                    if (alivePeers != null) {

                        var ss = StateServerFactory.getStateServer();
                        // 读一致性 
                        ss.readIndexState( new GenericClosure<State>() {

                            @Override
                            public void run(Status status) {
                                var state = this.getValue();

                                alivePeers.forEach((peer) -> {
                                    WorkerState ws = state.getWorkers().get(peer);
                                    if (ws != null) {
                                        // 
                                        var canDealTasks = MAX_TASKS - ws.getTaskQueueSize();
                                        log.debug("cap :{} peer: {}", canDealTasks, peer);
                                        if (canDealTasks <= 0) {
                                            return;
                                        }
                                        ws.setUpdateAt(Instant.now());
                                        ws.setTaskQueueSize(MAX_TASKS);
    
                                        var request = new PacketsRequest();
                                        for (int i = 0; i < canDealTasks; i++) {
                                            var p = Any.pack(BacktrackingFlowOuterClass.BacktrackingFlow
                                                    .newBuilder()
                                                    .setTableId(10086)
                                                    .build());
                                            request.getPackets().add(p);
                                        }
    
                                        Operate.CallOperate(new Operate(OperateType.PUT, ws), new GenericClosure<Operate>() {
                                            @Override
                                            public void run(Status status) {
    
                                                log.info("{}", status);
                                                try {
                                                    ss.getRpcClient().invokeAsync(peer.getEndpoint(), request,
                                                            new InvokeCallback() {
                                                                @Override
                                                                public void complete(Object result, Throwable err) {
                                                                    log.info("PacketsRequest: {}", result);
                                                                }
                                                            }, 5000);
                                                } catch (InterruptedException | RemotingException e) {
                                                    log.info("error send packets {}", e.toString());
                                                }
                                            }
                                        });
                                    }
                                });
                            }
                            
                        } );
                    }

                    Thread.sleep(5000);
                }
            } catch (InterruptedException e) {
                log.info("{}", e.toString());
            }

        }
    });

    public static Thread getMasterExecute() {
        return masterExecute;
    }

    public static void Init() {

    }

}
